﻿/// <reference path="RCommon-1.5.3.d.ts" />
/*!***********************************
插件名称：RCommon
当前版本：1.5.3
更新日期：2022.8.23
插件官网：http://www.rplugin.com
Copyright (c) 2022 RUI, Robert Wu
Released under the MIT license
*************************************/

// #region 扩展Function

(function () {
    var p = Function.prototype;

    // 继承
    if (!p.extend) {
        p.extend = function (superClass) {
            var F = function () { };
            F.prototype = superClass.prototype;
            this.prototype = new F();
            this.prototype.constructor = this;
            this.superClass = superClass;
            if (superClass.prototype.constructor == Object.prototype.constructor) {
                superClass.prototype.constructor = superClass;
            }
            return this;
        };
    }
})();

// #endregion


// #region 扩展String

(function () {
    var p = String.prototype;

    // 将字符串转换成整数并返回（当转换失败时返回defaultNum）
    if (!p.parseInt) {
        p.parseInt = function (defaultValue) {
            var number = parseInt(this);
            if (isNaN(number)) number = defaultValue;
            return number;
        };
    }

    // 将字符串转换成浮点数并返回（当转换失败时返回defaultNum）
    if (!p.parseFloat) {
        p.parseFloat = function (defaultValue) {
            var number = parseFloat(this);
            if (isNaN(number)) number = defaultValue;
            return number;
        };
    }

    // 清除前置和后置的字符
    p.trim = function (target) {
        if (!this) return "";
        if (!target) {
            return this.replace(/(^\s*)|(\s*$)/g, '');
        }
        else {
            if (typeof target != "string" || target.length != 1)
                throw "无效的target";
            var res = "" + this;
            while (res.length > 0 && res[0] == target)
                res = res.substr(1);
            while (res.length > 0 && res[res.length - 1] == target)
                res = res.substr(0, res.length - 1);
            return res;
        }
    };

    // 清除前置的字符
    p.trimStart = function (target) {
        if (!this) return "";
        if (!target) {
            return this.replace(/(^\s*)/g, '');
        }
        else {
            if (typeof target != "string" || target.length != 1)
                throw "无效的target";
            var res = "" + this;
            while (res.length > 0 && res[0] == target)
                res = res.substr(1);
            return res;
        }
    };

    // 清除后置的字符
    p.trimEnd = function (target) {
        if (!this) return "";
        if (!target) {
            return this.replace(/(\s*$)/g, '');
        }
        else {
            if (typeof target != "string" || target.length != 1)
                throw "无效的target";
            var res = "" + this;
            while (res.length > 0 && res[res.length - 1] == target)
                res = res.substr(0, res.length - 1);
            return res;
        }
    };

    // 替换所有匹配
    if (!p.replaceAll) {
        p.replaceAll = function (reallyDo, replaceWith, ignoreCase) {
            if (!RegExp.prototype.isPrototypeOf(reallyDo)) {
                return this.replace(new RegExp(reallyDo, (ignoreCase ? "gi" : "g")), replaceWith);
            } else {
                return this.replace(reallyDo, replaceWith);
            }
        };
    }

    // 将字符串中的首个字符（如果是英文字符的话）转换为大写
    //if (!p.capFirst) {
    //    p.capFirst = function () {
    //        if (!this) return this;
    //        return this[0].toUpperCase() + this.substr(1);
    //    };
    //}

    // 将字符串转换成驼峰写法（去掉'-'，并把'-'后面的第一个字母改成大写）
    // 例：输入"background-image"，返回"backgroundImage"
    if (!p.camelize) {
        p.camelize = function () {
            if (!this) return "";
            return this.replace(/\-(\w)/ig, function (B, A) {
                return A.toUpperCase();
            });
        };
    }

    // 进行HTML编码
    // 例：传入"<test>\n"，返回"&lt;test&gt;<br/>"
    if (!p.encodeHtml) {
        p.encodeHtml = function () {
            if (!this) return "";
            var temp = this;

            temp = temp.replace(/&/g, "&amp;");

            temp = temp.replace(/ /g, "&nbsp;");

            temp = temp.replace(/</g, "&lt;");
            temp = temp.replace(/>/g, "&gt;");

            temp = temp.replace(/\"/g, "&quot;");
            temp = temp.replace(/\'/g, "&#39;");

            temp = temp.replace(/￠/g, "&cent;");
            temp = temp.replace(/£/g, "&pound;");
            temp = temp.replace(/¥/g, "&yen;");
            temp = temp.replace(/€/g, "&euro;");

            temp = temp.replace(/§/g, "&sect;");

            temp = temp.replace(/©/g, "&copy;");
            temp = temp.replace(/®/g, "&reg;");
            temp = temp.replace(/™/g, "&trade;");

            temp = temp.replace(/×/g, "&times;");
            temp = temp.replace(/÷/g, "&divide;");

            temp = temp.replace(/\n/g, "<br/>");

            return temp;
        };
    }

    // 进行HTML解码
    // 例：传入"&lt;test&gt;<br/>"，返回"<test>\n"
    if (!p.decodeHtml) {
        p.decodeHtml = function () {
            if (!this) return "";
            var temp = this;

            temp = temp.replace(/&nbsp;/g, " ");

            temp = temp.replace(/&lt;/g, "<");
            temp = temp.replace(/&gt;/g, ">");

            temp = temp.replace(/&quot;/g, "\"");
            temp = temp.replace(/&#39;/g, "\'");

            temp = temp.replace(/&cent;/g, "￠");
            temp = temp.replace(/&pound;/g, "£");
            temp = temp.replace(/&yen;/g, "¥");
            temp = temp.replace(/&euro;/g, "€");

            temp = temp.replace(/&sect;/g, "§");

            temp = temp.replace(/&copy;/g, "©");
            temp = temp.replace(/&reg;/g, "®");
            temp = temp.replace(/&trade;/g, "™");

            temp = temp.replace(/&times;/g, "×");
            temp = temp.replace(/&divide;/g, "÷");

            temp = temp.replace(/<br \/>/g, "\n");
            temp = temp.replace(/<br\/>/g, "\n");
            temp = temp.replace(/<br>/g, "\n");

            temp = temp.replace(/&amp;/g, "&");

            return temp;
        };
    }

    // 将字符串中的双引号编码成字符实体"&amp;quot;"
    if (!p.encodeQuot) {
        p.encodeQuot = function () {
            return this.replaceAll("\"", "&quot;");
        };
    }

    // escape
    if (!p.escape) {
        p.escape = function () {
            return escape(this);
        };
    }

    // unescape
    if (!p.unescape) {
        p.unescape = function () {
            return unescape(this);
        };
    }

    // 判断当前字符串是否以指定的字符串为开头
    if (!p.startsWith) {
        p.startsWith = function (target) {
            if (!this) return false;
            if (typeof target != "string" || !target)
                throw "无效的target";
            if (target.length > this.length) return false;
            return this.substr(0, target.length) == target;
        };
    }

    // 判断当前字符串是否以指定的字符串为结尾
    if (!p.endsWith) {
        p.endsWith = function (target) {
            if (!this) return this;
            if (typeof target != "string" || !target)
                throw "无效的target";
            if (target.length > this.length) return false;
            return this.substr(this.length - target.length) == target;
        };
    }

    // 按最大长度省略
    if (!p.ellipsis) {
        p.ellipsis = function (maxlength, ellipsisString) {
            if (!this || this.length <= maxlength) return "" + this;
            if (!ellipsisString) ellipsisString = "...";
            return this.substr(0, maxlength) + ellipsisString;
        };
    }
})();

// #endregion


// #region 扩展Number

(function () {
    var p = Number.prototype;

    // 保留到小数点后roundDigit位
    // roundDigit（可选）：保留到小数点后多少位，默认两位
    // fivein（可选）：是否进行四舍五入，默认进行
    if (!p.round) {
        p.round = function (roundDigit, fivein) {
            if (this < 0) return -this.round(-this, roundDigit, fivein);
            if (typeof (roundDigit) == "undefined") return this.round(this, 2, fivein);
            return parseInt((this * Math.pow(10, roundDigit) + ((fivein == false) ? 0 : 0.5))) / Math.pow(10, roundDigit);
        };
    }

    // 判断该数字是否存在小数
    if (!p.isDecimal) {
        p.isDecimal = function () {
            var intthis = parseInt(this);
            return (this - intthis != 0);
        };
    }
})();

// #endregion


// #region 扩展Array

(function () {
    var p = Array.prototype;

    // 查找指定对象在数组中第一次出现的下标
    if (!p.indexOf) {
        p.indexOf = function (obj) {
            var index = -1;
            for (var i = 0; i < this.length; i++) {
                if (this[i] === obj) {
                    index = i;
                    break;
                }
            }
            return index;
        };
    }

    // 查找符合指定过滤条件的第一个元素的下标
    if (!p.indexOfFn) {
        p.indexOfFn = function (fn) {
            if (typeof fn != "function") return -1;
            var index = -1;
            for (var i = 0; i < this.length; i++) {
                if (fn(this[i])) {
                    index = i;
                    break;
                }
            }
            return index;
        };
    }

    // 获取指定下标的元素，若不存在则返回指定默认值
    if (!p.get) {
        p.get = function (index, defaultValue) {
            if (index < 0) index += this.length;
            if (index < 0 || index >= this.length) return defaultValue;
            return this[index];
        };
    }

    // 遍历数组对每一个元素执行指定函数，若函数返回false则停止遍历
    if (!p.each) {
        p.each = function (fn) {
            if (typeof fn != "function") return this;
            for (var i = 0; i < this.length; i++) {
                var item = this[i];
                if (fn.call(item, i, item) == false)
                    break;
            }
            return this;
        };
    }

    // 无重复地添加一个元素到末尾
    if (!p.add) {
        p.add = function (obj) {
            var index = this.indexOf(obj);
            if (index < 0)
                this.push(obj);
            return this;
        };
    }

    // 删除指定元素
    if (!p.remove) {
        p.remove = function (obj) {
            this.removeBy(function (item) {
                return obj === item;
            });
            return this;
        };
    }

    // 删除指定下标的元素
    if (!p.removeAt) {
        p.removeAt = function (index) {
            if (index < 0 || index >= this.length) return this;
            this.splice(index, 1);
            return this;
        };
    }

    // 获取第一个元素
    if (!p.first) {
        p.first = function (defaultValue) {
            return this.get(0, defaultValue);
        };
    }

    // 获取最后一个元素
    if (!p.last) {
        p.last = function (defaultValue) {
            return this.get(this.length - 1, defaultValue);
        };
    }

    // 根据自定义条件筛选出符合条件的元素的数组
    var where = function (fn) {
        var res = [];
        if (typeof fn != "function") return res;
        for (var i = 0; i < this.length; i++) {
            if (fn(this[i])) res.push(this[i]);
        }
        return res;
    };
    if (!p.select) p.select = where;
    if (!p.where) p.where = where;

    // 根据自定义条件筛选出符合条件的第一个元素
    if (!p.selectFirst) {
        p.selectFirst = function (fn, defaultValue) {
            var res = defaultValue;
            if (typeof fn != "function") return res;
            for (var i = 0; i < this.length; i++) {
                if (fn(this[i])) { res = this[i]; break; }
            }
            return res;
        };
    }

    // 插入一个元素到指定位置
    if (!p.insert) {
        p.insert = function (item, index) {
            if (typeof index == "undefined") index = this.length - 1;
            this.splice(index, 0, item);
            return this;
        };
    }

    // 将当前数组中的每个元素处理成新的元素并返回新元素的数组
    var remap = function (fn) {
        if (typeof fn != "function") return this;
        var res = [];
        for (var i = 0; i < this.length; i++) {
            var item = this[i];
            var newitem = fn.call(item, item, i, this);
            res.push(newitem);
        }
        return res;
    };
    if (!p.map) p.map = remap;
    if (!p.remap) p.remap = remap;

    // 指定一个判断条件，将符合该条件的所有项移除
    if (!p.removeBy) {
        p.removeBy = function (fn) {
            for (var i = this.length - 1; i >= 0; i--) {
                if (fn(this[i]))
                    this.splice(i, 1);
            }
            return this;
        };
    }
})();

// #endregion


// #region 扩展jQuery

if (typeof window.jQuery != "undefined") {
    // 创建一个元素
    $.create = function (tagName, className, html) {
        if (!tagName) return $();
        var newJdom = $(document.createElement(tagName));
        if (typeof className != "undefined") newJdom.addClass(className);
        if (typeof html != "undefined") newJdom.html(html);
        return newJdom;
    };

    // 获取指定名称的属性的值，若不存在则返回空字符串
    $.prototype.attrstr = function (name) {
        var value = this.attr(name);
        if (typeof value == "undefined") return "";
        return "" + value;
    };

    // 根据指定值显示/隐藏元素
    $.prototype.showhide = function (value) {
        if (value) this.show();
        else this.hide();
        return this;
    };

    // 根据指定值添加/移除类名
    $.prototype.addRemoveClass = function (className, value) {
        if (value) this.addClass(className);
        else this.removeClass(className);
        return this;
    };

    // 判断指定的坐标是否在当前元素内
    $.prototype.isInside = function (x, y) {
        var region = {};
        var offset = this.offset();
        region.top = offset.top;
        region.left = offset.left;
        region.bottom = region.top + this.outerHeight();
        region.right = region.left + this.outerWidth();
        return x > region.left && x < region.right && y > region.top && y < region.bottom;
    };
}

// #endregion


// #region RCommon [2021.1.12]

window.RCommon = {};

// 在指定window对象中查找指定名称的window对象，若找不到则返回null，若指定名称为空则直接返回顶层window
// 传入window,name
// 传入name
RCommon.getWindowByName = function (a, b) {
    if (typeof (b) == "undefined") return getWindowByName(top.window, a);
    if (a && a.frames) {
        if (b == "") return top.window;
        if (a.name == b) return a;
        for (var i = 0; i < a.frames.length; i++) {
            var res = getWindowByName(a.frames[i].window, b);
            if (res != null) return res;
        }
        return null;
    }
    else {
        alert("传入的不是window对象");
        return null;
    }
};

// 遍历当前顶层页面及其内含的所有iframe页面执行操作
RCommon.eachWindow = function (a, b) {
    var w = null;
    var fn = null;
    if (typeof b == "undefined") {
        w = top.window;
        fn = a;
    }
    else {
        w = a;
        fn = b;
    }
    if (!w || !fn) return;

    var each = function (w, fn) {
        try {
            var res = fn(w);
            if (res === false) return;
            if (w.frames.length && w.frames.length > 0) {
                for (var i = 0; i < w.frames.length; i++) {
                    each(w.frames[i], fn);
                }
            }
        } catch (e) {

        }
    };
    each(w, fn);
    return this;
};

// 生成4个16位（0~f）随机码
RCommon.random16_4 = function () {
    return (((1 + Math.random()) * 0x10000) | 0).toString(16).substring(1);
};

// 生成一个随机的GUID
RCommon.guid = function () {
    var S4 = this.random16_4;
    return (S4() + S4() + "-" + S4() + "-" + S4() + "-" + S4() + "-" + S4() + S4() + S4());
};

// 获取dom元素（原生js对象）的指定styleName样式的最顶层值
// styleName请使用标准css写法
// 例：请使用"background-color"，不要使用"backgroundColor"
RCommon.style = function (dom, styleName) {
    try {
        var value;
        if (styleName == "float")
            document.defaultView ? styleName = 'float' : styleName = 'styleFloat';
        value = dom.style[styleName] || dom.style[styleName.camelize()];
        if (!value) {
            if (document.defaultView && document.defaultView.getComputedStyle) {
                var _css = document.defaultView.getComputedStyle(dom, null);
                value = _css ? _css.getPropertyValue(styleName) : null;
            }
            else {
                if (dom.currentStyle)
                    value = dom.currentStyle[styleName.camelize()];
            }
        }

        if (value == "auto" && ["width", "height"].indexOf(styleName) >= 0 && dom.style.display != "none")
            value = dom["offset" + styleName.capFirst()] + "px";

        if (styleName == "opacity") {
            try {
                value = dom.filters['DXImageTransform.Microsoft.Alpha'].opacity;
                value = value / 100;
            } catch (e) {
                try {
                    value = dom.filters('alpha').opacity;
                } catch (err) { }
            }
        }

        return value == "auto" ? null : value;
    } catch (e) {
        return "";
    }
};

// 获取分页数（当共有recordCount条记录，每页显示pageSize条记录时，计算一共有几页）
RCommon.getPage = function (recordCount, pageSize) {
    if (isNaN(recordCount)) return 0;
    if (isNaN(pageSize)) return 0;
    var res = parseInt(recordCount / pageSize);
    var res2 = parseInt(recordCount % pageSize);
    if (res2 > 0) res++;
    return res;
};

// 判断一个变量的数据类型
RCommon.getType = function (input) {
    var t = typeof input;
    if (t == "undefined" || input === null) return "undefinedOrNull";
    if (typeof input.length == "number" && typeof input.jquery != "undefined") return "jquery";
    if (typeof input.nodeType == "number" && typeof input.nodeName == "string") return "dom";
    if (t == "boolean" || t == "number" || t == "string") return "simpleData";
    if (typeof input.length == "number") return "array";
    return t;
};

// 克隆一个数据
RCommon.clone = function (input, settings) {
    var pthis = this;
    if (!settings) {
        settings = {
            addKeyToObject: false,
            cloneDom: true
        };
    }
    var type = pthis.getType(input);
    switch (type) {
        case "array":
            {
                var res = [];
                foreach(input, function (item) {
                    res.push(pthis.clone(item, settings));
                });
                return res;
            }
        case "simpleData":
            return input;
        case "object":
            {
                var res = {};

                if (settings.addKeyToObject && typeof input["__key"] == "undefined")
                    res["__key"] = input["__key"] = RUI.guid();

                for (var key in input) {
                    if (settings.skipKeys && settings.skipKeys.indexOf(key) >= 0)
                        continue;
                    res[key] = pthis.clone(input[key], settings);
                }
                return res;
            }
        case "dom":
        case "jquery":
            return settings.cloneDom !== false ? input : null;
        default:
            return input;
    }
};

// 判断输入的参数是否为一个纯粹的对象
RCommon.isPlainObject = function (obj) {
    if (typeof obj !== 'object' || obj === null) return false;
    var proto = obj;
    while (Object.getPrototypeOf(proto) !== null)
        proto = Object.getPrototypeOf(proto);
    return Object.getPrototypeOf(obj) === proto;
};

// 遍历数组执行动作
window.foreach = function (arr, fn) {
    if (!fn) return;
    if (typeof fn != "function") return;
    if (!arr || !arr.length) return;
    if (typeof arr.length != "number") return;
    if (arr.length <= 0) return;
    for (var index = 0; index < arr.length; index++) {
        var item = arr[index];
        var res = fn.apply(item, [item, index]);
        if (res === false)
            break;
    }
};

// #endregion


// #region RUrl [2021.1.12]

window.RUrl = {};

// 获取指定name的url参数
RUrl.queryString = function (a, b) {
    var u = "";
    var name = "";
    if (typeof b == "undefined") {
        u = "" + window.location.href;
        name = a;
    }
    else {
        u = a;
        name = b;
    }

    if (!name) return "";
    name = name.toLowerCase();

    var index1 = u.indexOf('#');
    if (index1 >= 0) u = u.substring(0, index1);

    var index2 = u.indexOf('?');
    if (index2 >= 0) {
        var paramstr = u.substring(index2 + 1);
        var arr = paramstr.split('&');
        for (var i = 0; i < arr.length; i++) {
            var item = arr[i];
            if (!item) continue;
            var arr2 = item.split('=');
            var pname = arr2[0].toLowerCase();
            if (pname == name) {
                return decodeURIComponent(arr2[1]);
            }
        }
    }
    return "";
};

// 获取指定url地址中的所有url参数
RUrl.getAllParams = function (url) {
    var u = "" + (url ? url : window.location.href);
    var index1 = u.indexOf('#');
    if (index1 >= 0) u = u.substring(0, index1);

    var params = {};
    var index2 = u.indexOf('?');
    if (index2 >= 0) {
        var paramstr = u.substring(index2 + 1);
        var arr = paramstr.split('&');
        for (var i = 0; i < arr.length; i++) {
            var item = arr[i];
            if (!item) continue;
            var arr2 = item.split('=');
            var pname = arr2[0];
            var pvalue = decodeURIComponent(arr2[1]);
            params[pname] = pvalue;
        }
    }
    return params;
};

// 修改URL参数
RUrl.changeParam = function (url, a, b) {
    var map = null;
    if (typeof b == "undefined") {
        if (RCommon.isPlainObject(a))
            map = RCommon.clone(a);
    }
    else if (typeof a == "string") {
        map = {};
        map[a] = b;
    }
    if (!map) return url;

    var u = "" + url;

    // 获取并移除hash部分
    var index1 = u.indexOf('#');
    var hashpart = "";
    if (index1 >= 0) {
        hashpart = u.substr(index1);
        u = u.substring(0, index1);
    }

    // 获取并移除url参数部分
    var index2 = u.indexOf('?');
    var params;
    if (index2 >= 0) {
        params = this.getAllParams(u);
        u = u.substring(0, index2);
    }
    else {
        params = {};
    }

    // 更新url参数
    for (var name in map) {
        var value = map[name];
        if (value === null || typeof value == "undefined")
            delete params[name];
        else
            params[name] = "" + value;
    }

    // 生成新的url
    var querypart = "";
    for (var name in params) {
        var value = params[name];
        if (querypart)
            querypart += "&";
        else
            querypart += "?";
        querypart += name + "=" + encodeURIComponent(value);
    }

    var res = u;
    res += querypart;
    res += hashpart;
    return res;
};

// 获取或设置hash
RUrl.hash = function (a, b) {
    var getData = function () {
        var hash = location.hash;
        if (hash[0] == "#") hash = hash.substring(1);
        if (hash[0] == "?") hash = hash.substring(1);
        var arr = hash.split(";");
        var data = {};
        foreach(arr, function (item) {
            if (!item) return;
            var temp = item.split("=");
            if (temp.length != 2) return;
            data[temp[0]] = decodeURIComponent(temp[1]);
        });
        return data;
    };
    var setData = function (data) {
        var str = "?";
        var i = 0;
        for (var name in data) {
            var value = data[name];
            var valuestr = "";
            switch (typeof value) {
                case "string":
                    valuestr = value;
                    break;
                case "number":
                    valuestr = "" + value;
                    break;
                default:
                    valuestr = JSON.stringify(value);
                    break;
            }
            if (i != 0) str += ";";
            str += name + "=" + encodeURIComponent(valuestr);
            i++;
        }
        location.hash = str;
    };

    var data = getData();
    if (typeof a == "undefined") {
        return data;
    }
    else if (typeof a == "string") {
        if (typeof b == "undefined") {
            return data[a];
        }
        else {
            if (b == null)
                delete data[a];
            else
                data[a] = b;
            setData(data);
            return this;
        }
    }
    else if (RCommon.isPlainObject(a)) {
        for (var name in a) {
            var value = a[name];
            if (value === null)
                delete data[name];
            else
                data[name] = value;
        }
        setData(data);
        return this;
    }
    return this;
};

// #endregion


// #region RCookie [2021.6.9]

window.RCookie = {
    // 是否要对来自localhost的请求进行端口区分的特殊处理（默认是）
    distinguishLocalhostPort: true
};

// 获取根据localhost进行端口区分后的新的键名
RCookie.getDistinguishedName = function (name) {
    if (this.distinguishLocalhostPort && window.location.hostname == "localhost")
        return window.location.port + "_" + name;
    return name;
};

// 添加指定name的cookie，值为value
RCookie.add = function (name, value, options) {
    if (!name) return;
    if (!options) options = {};

    name = this.getDistinguishedName("" + name);

    if (typeof value == "undefined" || value === null)
        value = "";
    else if (typeof value != "string")
        value = "" + value;

    var cookiestr;
    if (options.doEscape !== false)
        cookiestr = name + "=" + escape(value) + ";";
    else
        cookiestr = name + "=" + value + ";";

    // 设置有效时长
    if (typeof options.minutes == "number") {
        var d = new Date();
        d.setTime(d.getTime() + (options.minutes * 60 * 1000));
        cookiestr += "expires=" + d.toUTCString() + ";";
    }

    // 设置域
    if (options.domain)
        cookiestr += "domain=" + options.domain + ";";

    // 设置路径
    cookiestr += "path=" + (options.path || "/") + ";";

    document.cookie = cookiestr;
    return this;
};

// 获取指定name的cookie的值，获取不到时返回空字符串
RCookie.get = function (name) {
    if (!name) return "";
    var newname = this.getDistinguishedName("" + name);

    var arr, reg = new RegExp("(^| )" + newname + "=([^;]*)(;|$)");
    if (arr = document.cookie.match(reg))
        return unescape(arr[2]);
    else
        return "";
};

// 删除指定name的cookie
RCookie.remove = function (name) {
    if (!name) return;
    this.add(name, "", { minutes: -10 });
    return this;
};

// 获取当前域所有的cookie的数组
RCookie.getall = function () {
    var res = [];
    var cookies = document.cookie.split(";");
    var prefix = (this.distinguishLocalhostPort && window.location.hostname == "localhost") ? prefix = window.location.port + "_" : "";
    for (var i = 0; i < cookies.length; i++) {
        var cookie = cookies[i];
        if (!cookie) continue;
        cookie = cookie.trim();
        var arr = cookie.split("=");
        if (arr.length < 2) continue;
        var name = arr[0];
        if (prefix && name.substr(0, prefix.length) != prefix) continue;
        res.push({
            name: name.substr(prefix.length),
            value: unescape(arr[1])
        });
    }
    return res;
};

// 删除当前域所有的cookie
RCookie.clear = function () {
    var list = this.getall();
    for (var i = 0; i < list.length; i++) {
        this.remove(list[i].name);
    }
    return this;
};

// #endregion


// #region RLocalStorage [2021.1.12]

window.RLocalStorage = {
    // 是否要对来自localhost的请求进行端口区分的特殊处理（默认是）
    distinguishLocalhostPort: true
};

// 获取根据localhost进行端口区分后的新的键名
RLocalStorage.getDistinguishedName = function (name) {
    if (this.distinguishLocalhostPort && window.location.hostname == "localhost")
        return window.location.port + "_" + name;
    return name;
};

// 添加指定name的localStorage
RLocalStorage.add = function (name, value) {
    if (!name) return;
    var newname = this.getDistinguishedName("" + name);

    if (typeof value == "undefined" || value === null)
        value = "";
    else if (typeof value != "string")
        value = "" + value;

    window.localStorage[newname] = value;
    return this;
};

// 删除指定name的localStorage
RLocalStorage.remove = function (name) {
    if (!name) return;
    var newname = this.getDistinguishedName("" + name);
    window.localStorage.removeItem(newname);
    return this;
};

// 获取指定name的localStorage的值，获取不到时返回空字符串
RLocalStorage.get = function (name) {
    if (!name) return "";
    var newname = this.getDistinguishedName("" + name);
    return window.localStorage[newname];
};

// 获取当前域所有的localStorage的数组
RLocalStorage.getall = function () {
    var res = [];
    var prefix = (this.distinguishLocalhostPort && window.location.hostname == "localhost") ? prefix = window.location.port + "_" : "";
    for (var name in window.localStorage) {
        if (prefix && name.substr(0, prefix.length) != prefix) continue;
        res.push({
            name: name.substr(prefix.length),
            value: window.localStorage[name]
        });
    }
    return res;
};

// 删除当前域所有的localStorage
RLocalStorage.clear = function () {
    var pthis = this;
    var list = pthis.getall();
    foreach(list, function (item) {
        pthis.remove(item.name);
    });
    return pthis;
};

// #endregion


// #region RDebug [2021.3.5]

window.RDebug = {
    _t1: new Date().getTime(),
    _t2: new Date().getTime()
};

// 记录时间
RDebug.lt = function () {
    this._t1 = this._t2;
    this._t2 = new Date().getTime();
    return this;
};

// 获取耗时
RDebug.t = function () {
    return this._t2 - this._t1;
};

// 获取耗时的提示文字
RDebug.ts = function () {
    var res1 = this._t2 - this._t1;
    var res2 = res1 / 1000;
    var res3 = parseInt((res2 * 100) / 60) / 100;
    return "耗时为" + res1 + "毫秒\n即" + res2 + "秒\n约" + res3 + "分钟";
};

// 弹框提示耗时
RDebug.tsa = function () {
    alert(this.ts());
    return this;
};

// 快速输出一段console日志
RDebug.log = function (text, color) {
    if (!color || !!window.ActiveXObject || "ActiveXObject" in window)
        console.log(text);
    else
        console.log("%c " + text, "color:" + color + ";");
};

// #endregion


// #region RDomMaker [2021.6.10]

// 元素生成器
window.RDomMaker = function (name, dommap) {
    if (typeof window.jQuery == "undefined")
        throw "RDomMaker基于jQuery，请先引入jQuery";
    // 名称
    this.name = name;
    // 图纸内容
    this.map = dommap;
};

(function () {
    var p = RDomMaker.prototype;

    // 遍历map对每一项执行指定的函数
    p.eachDoms = function (fn) {
        var pthis = this;
        for (var name in pthis.map) {
            var data = pthis.map[name];
            if (fn) fn(name, data);
        }
        return pthis;
    };

    // 初始化map中的所有项到容器
    p.initDoms = function (doms, settings) {
        var pthis = this;
        for (var name in pthis.map) {
            var data = pthis.map[name];
            if (data.initcreate === false) continue;
            pthis._createDom(name, data, doms, settings);
        }
        return pthis;
    };

    // 创建指定名称的元素到容器
    p.createDom = function (name, doms, settings) {
        return this._createDom(name, this.map[name], doms, settings);
    };

    // 创建指定名称数组的元素到容器
    p.createDoms = function (names, doms, settings) {
        var pthis = this;
        foreach(names, function (name) {
            var data = pthis.map[name];
            pthis._createDom(name, data, doms, settings);
        });
        return pthis;
    };

    // 创建指定名称数组的元素
    p._createDom = function (name, data, doms, settings) {
        var jdom = $();

        // 没有参数
        if (!name || !data) return jdom;

        // 已有该元素
        jdom = doms[name];
        if (jdom && jdom.length > 0) return jdom;

        // 创建元素
        var target = null;
        if (data.checkexist)
            target = data.checkexist(name, data, doms, settings);
        if (target)
            jdom = target;
        else
            jdom = $.create(data.tag || "div");

        // 设置属性
        {
            var classname = name;
            if (data.prefix !== false && settings.prefixname)
                classname = settings.prefixname + classname;
            if (data.addNameToClass !== false)
                classname = this.name + classname;
            if (data.postfix !== false && settings.postfixname)
                classname = classname + settings.postfixname;
            jdom.addClass(classname);
            if (data.extraclass) jdom.addClass(data.extraclass);

            if (typeof data.html != "undefined") jdom.html(data.html);
            if (data.hidden) jdom.hide();

            if (data.attrs) {
                for (var attrname in data.attrs) {
                    if (typeof jdom.attr(attrname) == "undefined")
                        jdom.attr(attrname, data.attrs[attrname]);
                }
            }
        }

        // 添加到指定位置
        {
            var getTarget = function (name) {
                if (name == "jdom" && settings.root)
                    return settings.root;
                if (doms[name])
                    return doms[name];
                return null;
            };
            if (data.insertBefore) {
                var target = getTarget(data.insertBefore);
                if (target) jdom.insertBefore(target);
            }
            else if (data.insertAfter) {
                var target = getTarget(data.insertAfter);
                if (target) jdom.insertAfter(target);
            }
            else if (data.appendTo) {
                var target = getTarget(data.appendTo);
                if (target) jdom.appendTo(target);
            }
            else if (data.prependTo) {
                var target = getTarget(data.prependTo);
                if (target) jdom.prependTo(target);
            }
        }

        doms[name] = jdom;

        if (settings.makingDom) settings.makingDom(jdom, name, data);

        return jdom;
    };

    // 从容器中移除所有元素
    p.removeAll = function (doms) {
        var pthis = this;
        for (var name in pthis.map) {
            var jdom = doms[name];
            if (jdom) jdom.remove();
            doms[name] = $();
        }
        return pthis;
    };
})();

// #endregion


// #region RGlobalEvents [2019.5.9]

if (typeof window.jQuery != "undefined") {
    // 全局事件管理器
    window.RGlobalEvents = {
        // 处理函数集合
        _eventlist: {
            mousemove: [],
            mousedown: [],
            mouseup: [],
            resize: [],
            keydown: [],
            keyup: []
        },
        // 添加一个【鼠标移动事件】处理函数
        onMouseMove: function (event) {
            this._eventlist["mousemove"].push(event);
            return this;
        },
        // 添加一个【鼠标左键点下事件】处理函数
        onMouseDown: function (event) {
            this._eventlist["mousedown"].push(event);
            return this;
        },
        // 添加一个【鼠标左键松开事件】处理函数
        onMouseUp: function (event) {
            this._eventlist["mouseup"].push(event);
            return this;
        },
        // 添加一个【窗口大小改变事件】处理函数
        onResize: function (event) {
            this._eventlist["resize"].push(event);
            return this;
        },
        // 添加一个【键按下事件】处理函数
        onKeydown: function (event) {
            this._eventlist["keydown"].push(event);
            return this;
        },
        // 添加一个【键放开事件】处理函数
        onKeyup: function (event) {
            this._eventlist["keyup"].push(event);
            return this;
        },
        // 主动触发指定名称的事件
        triggerEvent: function (name, arg) {
            if (!this._eventlist[name]) return;
            for (var i = 0; i < this._eventlist[name].length; i++) {
                var event = this._eventlist[name][i];
                var res = event(arg);
                if (res === false) return false;
            }
        }
    };

    $(document).mousemove(function (e) {
        return RGlobalEvents.triggerEvent("mousemove", e);
    }).mousedown(function (e) {
        return RGlobalEvents.triggerEvent("mousedown", e);
    }).mouseup(function (e) {
        return RGlobalEvents.triggerEvent("mouseup", e);
    }).keydown(function (e) {
        return RGlobalEvents.triggerEvent("keydown", e);
    }).keyup(function (e) {
        return RGlobalEvents.triggerEvent("keyup", e);
    });

    $(window).resize(function (e) {
        RGlobalEvents.triggerEvent("resize", e);
    });
}

// #endregion


// #region RSolar [2021.1.12]

// 阳历工具
window.RSolar = {
    // 阳历节日数据
    festivals: {
        1: {
            1: ["元旦"]
        },
        2: {
            2: ["世界湿地日"],
            10: ["国际气象节"],
            14: ["情人节"]
        },
        3: {
            1: ["国际海豹日"],
            5: ["学雷锋纪念日"],
            8: ["妇女节"],
            12: ["植树节", "孙中山逝世纪念日"],
            14: ["国际警察日"],
            15: ["消费者权益日"],
            17: ["中国国医节", "国际航海日"],
            21: ["世界森林日", "消除种族歧视国际日", "世界儿歌日"],
            22: ["世界水日"],
            24: ["世界防治结核病日"]
        },
        4: {
            1: ["愚人节"],
            7: ["世界卫生日"],
            22: ["世界地球日"]
        },
        5: {
            1: ["劳动节"],
            4: ["青年节"],
            8: ["世界红十字日"],
            12: ["国际护士节"],
            31: ["世界无烟日"]
        },
        6: {
            1: ["国际儿童节"],
            5: ["世界环境保护日"],
            26: ["国际禁毒日"]
        },
        7: {
            1: ["建党节", "香港回归纪念", "世界建筑日"],
            11: ["世界人口日"]
        },
        8: {
            1: ["建军节"],
            8: ["中国男子节", "中国父亲节"],
            15: ["抗日战争胜利纪念"]
        },
        9: {
            10: ["教师节"],
            18: ["九·一八事变纪念日"],
            20: ["国际爱牙日"],
            27: ["世界旅游日"]
        },
        10: {
            1: ["国庆节", "国际音乐日"],
            6: ["老人节"],
            24: ["联合国日"]
        },
        11: {
            10: ["世界青年节"],
            12: ["孙中山诞辰纪念"]
        },
        12: {
            1: ["世界艾滋病日"],
            3: ["世界残疾人日"],
            20: ["澳门回归纪念"],
            24: ["平安夜"],
            25: ["圣诞节"]
        }
    }
};

// 获取指定阳历年的指定阳历月有几天
RSolar.getDaysInMonth = function (year, month) {
    month = parseInt(month, 10);
    var temp = new Date(year, month, 0);
    return temp.getDate();
};

// 获取用于日历显示的指定年月的日期数据
RSolar.getDatesForCalendar = function (year, month) {
    // 上月
    var prevmonth = month == 1 ? 12 : month - 1;
    // 下月
    var nextmonth = month == 12 ? 1 : month + 1;
    // 该月最大有几天
    var maxDate = this.getDaysInMonth(year, month);
    // 前一个月最大有几天
    var maxDatePrev = this.getDaysInMonth(year, month - 1);
    // 第一天是星期几
    var firstDay = new Date(year, month - 1, 1).getDay();
    if (firstDay == 0) firstDay = 7;
    // 最后一天是星期几
    var lastDay = (firstDay + maxDate - 1) % 7;
    if (lastDay == 0) lastDay = 7;

    var arr = [];
    for (var i = 1; i < firstDay; i++) arr.unshift({ type: 2, year: prevmonth == 12 ? year - 1 : year, month: prevmonth, date: maxDatePrev - i + 1 });
    for (var i = 1; i <= maxDate; i++) arr.push({ type: 1, year: year, month: month, date: i });
    for (var i = 1; i + lastDay <= 7; i++) arr.push({ type: 3, year: nextmonth == 1 ? year + 1 : year, month: nextmonth, date: i });

    return arr;
};

// 获取指定阳历月日的阳历节日
RSolar.getFestivals = function (month, date) {
    var monfes = this.festivals[month];
    if (monfes) {
        var fess = monfes[date];
        if (fess && fess.length > 0) {
            return fess;
        }
        else {
            return [];
        }
    }
    else {
        return [];
    }
};

// #endregion


// #region RLunnar [2021.1.12]

// 阴历工具
window.RLunnar = {
    // 阴历节日数据
    festivals: {
        1: {
            1: ["春节"],
            15: ["元宵节"]
        },
        5: {
            5: ["端午节"]
        },
        7: {
            7: ["七夕情人节"],
            15: ["中元节", "盂兰盆节"]
        },
        8: {
            15: ["中秋节"]
        },
        9: {
            9: ["重阳节"]
        },
        12: {
            8: ["腊八节"]
        }
    },
    _calendarData: [
        0xA4B, 0x5164B, 0x6A5, 0x6D4, 0x415B5, 0x2B6, 0x957, 0x2092F, 0x497, 0x60C96,
        0xD4A, 0xEA5, 0x50DA9, 0x5AD, 0x2B6, 0x3126E, 0x92E, 0x7192D, 0xC95, 0xD4A,
        0x61B4A, 0xB55, 0x56A, 0x4155B, 0x25D, 0x92D, 0x2192B, 0xA95, 0x71695, 0x6CA,
        0xB55, 0x50AB5, 0x4DA, 0xA5B, 0x30A57, 0x52B, 0x8152A, 0xE95, 0x6AA, 0x615AA,
        0xAB5, 0x4B6, 0x414AE, 0xA57, 0x526, 0x31D26, 0xD95, 0x70B55, 0x56A, 0x96D,
        0x5095D, 0x4AD, 0xA4D, 0x41A4D, 0xD25, 0x81AA5, 0xB54, 0xB6A, 0x612DA, 0x95B,
        0x49B, 0x41497, 0xA4B, 0xA164B, 0x6A5, 0x6D4, 0x615B4, 0xAB6, 0x957, 0x5092F,
        0x497, 0x64B, 0x30D4A, 0xEA5, 0x80D65, 0x5AC, 0xAB6, 0x5126D, 0x92E, 0xC96,
        0x41A95, 0xD4A, 0xDA5, 0x20B55, 0x56A, 0x7155B, 0x25D, 0x92D, 0x5192B, 0xA95,
        0xB4A, 0x416AA, 0xAD5, 0x90AB5, 0x4BA, 0xA5B, 0x60A57, 0x52B, 0xA93, 0x40E95
    ],
    _madd: [0, 31, 59, 90, 120, 151, 181, 212, 243, 273, 304, 334],
    _tgString: "甲乙丙丁戊己庚辛壬癸",
    _dzString: "子丑寅卯辰巳午未申酉戌亥",
    _sxString: "鼠牛虎兔龙蛇马羊猴鸡狗猪",
    _numString: "一二三四五六七八九十",
    _monString: "正二三四五六七八九十冬腊"
};

RLunnar._getbit = function (m, n) {
    return (m >> n) & 1;
};

// 将阳历日期转换为阴历日期
RLunnar.parse = function (year, month, date) {
    var cYear, cMonth, cDay, TheDate;

    TheDate = new Date(year, month - 1, date);
    var total, m, n, k;
    var isEnd = false;
    var tmp = TheDate.getYear();
    if (tmp < 1900) {
        tmp += 1900;
    }
    total = (tmp - 1921) * 365 + Math.floor((tmp - 1921) / 4) + this._madd[TheDate.getMonth()] + TheDate.getDate() - 38;

    if (TheDate.getYear() % 4 == 0 && TheDate.getMonth() > 1) {
        total++;
    }
    for (m = 0; ; m++) {
        k = (this._calendarData[m] < 0xfff) ? 11 : 12;
        for (n = k; n >= 0; n--) {
            if (total <= 29 + this._getbit(this._calendarData[m], n)) {
                isEnd = true; break;
            }
            total = total - 29 - this._getbit(this._calendarData[m], n);
        }
        if (isEnd) break;
    }
    cYear = 1921 + m;
    cMonth = k - n + 1;
    cDay = total;
    if (k == 12) {
        if (cMonth == Math.floor(this._calendarData[m] / 0x10000) + 1) {
            cMonth = 1 - cMonth;
        }
        if (cMonth > Math.floor(this._calendarData[m] / 0x10000) + 1) {
            cMonth--;
        }
    }

    return {
        year: cYear,
        month: cMonth,
        date: cDay
    };
};

// 获取指定阳历年的天干
RLunnar.gettg = function (year) {
    return this._tgString.charAt((year - 4) % 10);
};

// 获取指定阳历年的地支
RLunnar.getdz = function (year) {
    return this._dzString.charAt((year - 4) % 10);
};

// 获取指定阳历年的生肖
RLunnar.getsx = function (year) {
    return this._sxString.charAt((year - 4) % 12);
};

// 获取指定阳历年的生肖下标
RLunnar.getsxnum = function (year) {
    return (year - 4) % 12;
};

// 获取指定阴历月的中文名称
RLunnar.getmonth = function (month) {
    var tmp = "";
    if (month < 1) {
        tmp += "(闰)";
        tmp += this._monString.charAt(-month - 1);
    } else {
        tmp += this._monString.charAt(month - 1);
    }
    tmp += "月";
    return tmp;
};

// 获取指定阴历日的中文名称
RLunnar.getdate = function (date) {
    var tmp = "";
    tmp += (date < 11) ? "初" : ((date < 20) ? "十" : ((date < 30) ? "廿" : "三十"));
    if (date % 10 != 0 || date == 10) {
        tmp += this._numString.charAt((date - 1) % 10);
    }
    return tmp;
};

// 获取指定阴历月日的阴历节日
RLunnar.getFestivals = function (month, date) {
    var monfes = this.festivals[month];
    if (monfes) {
        var fess = monfes[date];
        if (fess && fess.length > 0) {
            return fess;
        }
        else {
            return [];
        }
    }
    else {
        return [];
    }
};

// #endregion


// #region RAnimation [2021.9.10]

function RAnimation(duration, onFrame) {
    this.handler = null;
    this.startTime = null;
    this.duration = duration;
    this.onFrame = onFrame;
    this.interval = 20;
}

(function () {
    var p = RAnimation.prototype;

    // 开始动画
    p.start = function () {
        var pthis = this;
        if (pthis.handler) clearInterval(pthis.handler);
        pthis.startTime = new Date().getTime();
        pthis.handler = setInterval(function () {
            var now = new Date().getTime();
            var pastTime = now - pthis.startTime;
            if (pthis.onFrame) pthis.onFrame.call(pthis, pastTime);
            if (pastTime >= pthis.duration) {
                clearInterval(pthis.handler);
                pthis.handler = null;
            }
        }, pthis.inteval);
    };

    // 暂停动画
    p.pause = function () {
        clearInterval(this.handler);
        this.handler = null;
    };

    // 结束动画
    p.end = function () {
        if (this.onFrame) this.onFrame.call(this, this.duration);
        clearInterval(this.handler);
        this.handler = null;
    };
})();

// #endregion


// #region StringReader

function StringReader(str) {
    if (typeof str != "string") throw "StringReader构造参数错误";
    this.str = str;
    this.currentIndex = 0;
    this.ch = str[0] || null;
}
(function () {
    var p = StringReader.prototype;

    // 判断当前游标是否已到达结尾
    p.isEnd = function () {
        return this.currentIndex >= this.str.length;
    };

    // 向后移动游标指定位数
    p.move = function (count) {
        this.currentIndex += count;
        this.ch = this.str[this.currentIndex] || null;
    };

    // 向后移动游标1位
    p.moveNext = function () {
        this.move(1);
    };

    // 移动游标，向后查找指定的目标字符串
    // 直到目标字符串第一次出现或游标已到达结尾处为止
    // 返回是否找到
    p.moveToString = function (target) {
        var found = false;
        while (!this.isEnd()) {
            if (this.look(target.length) == target) {
                found = true;
                break;
            }
            this.moveNext();
        }
        return found;
    };

    // 不移动游标，向后查看指定个数个字符（不包含当前字符）
    p.peek = function (count) {
        return this.str.substr(this.currentIndex + 1, count);
    };

    // 不移动游标，向后查看指定个数个字符（包含当前字符）
    p.look = function (count) {
        return this.str.substr(this.currentIndex, count);
    };
})();

// #endregion


